use axum::http::{header, HeaderMap, StatusCode};
use axum::response::{Html, IntoResponse, Json, Response};
use serde::Serialize;

use crate::common::err::AppError;

use super::Result;

pub const CODE_SUCCESS: i8 = 0;
pub const CODE_FAIL: i8 = -1;

pub type HtmlResponse = Html<String>;
pub type RedirectResponse = (StatusCode, HeaderMap, ());

pub fn redirect(url: &str) -> Result<RedirectResponse> {
    let mut header = HeaderMap::new();
    header.insert(header::LOCATION, url.parse().unwrap());
    Ok((StatusCode::FOUND, header, ()))
}

#[derive(Debug, Serialize, Clone)]
pub struct AppResponse<T: Serialize> {
    pub code: Option<i8>,
    pub msg: Option<String>,
    pub data: Option<T>,
}

impl<T: Serialize> IntoResponse for AppResponse<T> {
    fn into_response(self) -> Response {
        Json(self).into_response()
    }
}

impl<T: Serialize> AppResponse<T> {
    pub fn from_data(data: T) -> Self {
        Self {
            code: Some(CODE_SUCCESS),
            msg: Some("success".to_string()),
            data: Some(data),
        }
    }

    pub fn from_error(err: AppError) -> Self {
        Self {
            code: Some(CODE_FAIL),
            msg: Some(err.to_string()),
            data: None,
        }
    }

    pub fn from_result(result: Result<T>) -> Self {
        match result {
            Ok(data) => Self::from_data(data),
            Err(err) => Self::from_error(err),
        }
    }
}





